'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.assertPluralForms = exports.assertTokens = exports.assertUnexpected = exports.assertQuantity = exports.assertGettextInstance = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _token = require('./token');

/**
 * Throw where the object[field] is not a function.
 *
 * @throws Error on object does not contain a function
 * @param {object} obj An object that should contain a function member
 * @param {string} field The name of the member expected to be a function
 * @param {string} message A title for the error
 */
var assertGettextInstance = exports.assertGettextInstance = function assertGettextInstance(obj, field, message) {
  var isValid = obj !== null && (typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' && field in obj && typeof obj[field] === 'function';
  if (!isValid) {
    throw new Error(message + ': Gettext instance is missing ' + field + '() member');
  }
};

/**
 * Throws where the candidate is not an integer.
 *
 * @throws Error on non integer
 * @param {*} candidate Possible integer
 * @param {string} message A title for the error
 */
var assertQuantity = exports.assertQuantity = function assertQuantity(candidate, message) {
  var isValid = typeof candidate === 'number' && !isNaN(candidate) && candidate % 1 === 0;
  if (!isValid) {
    throw new Error(message + ': Expected an integer quantity');
  }
};

/**
 * Throws where the candidate is not an empty list.
 *
 * @throws Error on non-empty list
 * @param {Array} candidate List of additional arguments
 * @param {string} message A title for the error
 */
var assertUnexpected = exports.assertUnexpected = function assertUnexpected(candidate, message) {
  var isValid = Array.isArray(candidate) && candidate.length === 0;
  if (!isValid) {
    throw new Error(message + ': Expected no additional arguments');
  }
};

/**
 * Throw on token problems.
 *
 * @throws Error on token problems
 * @param {Array.<{name :string, value: *}>} tokens A number of tokens
 * @param {string} message A title for the error
 */
var assertTokens = exports.assertTokens = function assertTokens(tokens, message) {
  var errors = (0, _token.calculateErrors)(tokens);
  if (errors.length) {
    throw new Error(message + ': ' + errors[0]);
  }

  var collisions = (0, _token.calculateCollisions)(tokens);
  if (collisions.length) {
    throw new Error(message + ': substitution with the same name must have the same value: ' + collisions);
  }
};

/**
 * Throw on incorrect number of plural forms.
 *
 * @throws Error on incorrect number of plural forms
 * @param {number|*} expected Expected number of plural forms, possibly non-numeric where disabled
 * @param {number} actual Actual number of plural forms found
 * @param {string} message A title for the error
 */
var assertPluralForms = exports.assertPluralForms = function assertPluralForms(expected, actual, message) {
  if (!isNaN(expected) && expected > 0 && actual !== expected) {
    throw new Error(message + ': expected ' + expected + ' plural forms, saw ' + actual);
  }
};