'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ngettextDefault = exports.gettextDefault = exports.factory = exports.ngettextFactory = exports.gettextFactory = undefined;

var _token = require('./token');

var _template = require('./template');

var _defaults = require('./defaults');

var _assert = require('./assert');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Gettext as a tagged template string interpolator.
 *
 * For the interpolator, substitutions should be objects with a single key-value pair. The key
 * gives the `key` and `name` for the token and the `value` gives the value. Substitutions with the
 * same `name` must have `value` or an Error with result.
 *
 * Where all `substitutions` are `string` then a `string` is returned. Otherwise an `Array` is
 * returned. This allows React elements as substitutions. Elements must have an explicit `key` or
 * they will be assigned one automatically.
 *
 * Failure to pass a `gettext` function will not result in an error, but no translation will
 * occur.
 *
 * Customisation of the `toToken` and `finaliseToken` functions is for advanced users only. Make
 * reference to the source code.
 *
 * @throws Error On substitutions with duplicate `name`
 * @param {{gettext: function}} [gettext] Optional gettext instance that contains gettext function
 * @param {function} [toToken] Optional custom token inference function yields {label, name, value}
 * @param {function} [finaliseToken] Optional transform of token to final value
 * @param {string} [NODE_ENV] Reserved for testing
 * @returns {function(array, ...string):array|string} Template string interpolator
 */
var gettextFactory = exports.gettextFactory = function gettextFactory() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$gettext = _ref.gettext,
      gettext = _ref$gettext === undefined ? { gettext: _defaults.defaultGettext } : _ref$gettext,
      _ref$toToken = _ref.toToken,
      toToken = _ref$toToken === undefined ? _token.defaultToToken : _ref$toToken,
      _ref$finaliseToken = _ref.finaliseToken,
      finaliseToken = _ref$finaliseToken === undefined ? _token.defaultFinaliseToken : _ref$finaliseToken,
      _ref$NODE_ENV = _ref.NODE_ENV,
      NODE_ENV = _ref$NODE_ENV === undefined ? process.env.NODE_ENV : _ref$NODE_ENV;

  return function (strings) {
    for (var _len = arguments.length, substitutions = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      substitutions[_key - 1] = arguments[_key];
    }

    // get a msgid template with sensible token names
    var tokens = substitutions.map(toToken);
    var msgid = (0, _template.getTemplate)(strings, tokens);

    // validate unless production
    if (NODE_ENV !== 'production') {
      var message = 'gettext`' + msgid + '`';
      (0, _assert.assertGettextInstance)(gettext, 'gettext', message);
      (0, _assert.assertTokens)(tokens, message);
    }

    // translate and substitute
    //  we need to call from the gettext parent object in case gettext() uses 'this'
    var msgstr = gettext.gettext(msgid);
    return (0, _template.makeSubstitutions)({ msgstr: msgstr, tokens: tokens, finaliseToken: finaliseToken });
  };
};

/**
 * NGettext as a tagged template string interpolator.
 *
 * The function is a factory and must be closed with a `quantity` value in order to create an
 * interpolator. Normally this `quantity` would be a number that indicates singular (quantity === 1)
 * or plural otherwise.
 *
 * For the interpolator, substitutions should be objects with a single key-value pair. The key
 * gives the `key` and `name` for the token and the `value` gives the value.
 *
 * Singular and plural forms are contained in the same template and delimited by `delimiter`, by
 * default this is the pipe `|` character.
 *
 * The underlying `ngettext` implementation will choose a single form. It is passed the delimited
 * strings followed by all arguments to the outer function. Validation of arguments is the
 * concern of `ngettext` and the user must either ensure that the `quantity` is singular or provide
 * an `ngettext` that performs validation.
 *
 * Where all `substitutions` are `string` then a `string` is returned. Otherwise an `Array` is
 * returned. This allows React elements as substitutions. Elements must have an explicit `key` or
 * they will be assigned one automatically.
 *
 * Failure to pass a `ngettext` function will not result in an error, but no translation will
 * occur. This degenerate case supports exactly 2 plural forms which may not suit some developers.
 * If you need more forms for development then pass explict `numPlural` and a custom `splitPlural`
 * implementation.
 *
 * Customisation of the `toToken` and `finaliseToken` functions is for advanced users only. Make
 * reference to the source code.
 *
 * @throws Error On substitutions with duplicate `name`, or on insufficent plural forms
 * @param {{ngettext: function}} [gettext] Optional gettext instance that contains ngettext function
 * @param {function} [toToken] Optional custom token inference function yields {label, name, value}
 * @param {function} [finaliseToken] Optional transform of token to final value
 * @param {function} [splitPlural] Optional split string to plural forms, yeilds Array
 * @param {Number} [numPlural] Optional expected number of plurals (expected delimiters + 1)
 * @param {string} [NODE_ENV] Reserved for testing
 * @returns {function(quantity:int):function} Factory for a template string interpolator
 */
var ngettextFactory = exports.ngettextFactory = function ngettextFactory() {
  var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref2$gettext = _ref2.gettext,
      gettext = _ref2$gettext === undefined ? { ngettext: _defaults.defaultNgettext } : _ref2$gettext,
      _ref2$toToken = _ref2.toToken,
      toToken = _ref2$toToken === undefined ? _token.defaultToToken : _ref2$toToken,
      _ref2$finaliseToken = _ref2.finaliseToken,
      finaliseToken = _ref2$finaliseToken === undefined ? _token.defaultFinaliseToken : _ref2$finaliseToken,
      _ref2$splitPlural = _ref2.splitPlural,
      splitPlural = _ref2$splitPlural === undefined ? _defaults.defaultSplitPlural : _ref2$splitPlural,
      _ref2$numPlural = _ref2.numPlural,
      numPlural = _ref2$numPlural === undefined ? 2 : _ref2$numPlural,
      _ref2$NODE_ENV = _ref2.NODE_ENV,
      NODE_ENV = _ref2$NODE_ENV === undefined ? process.env.NODE_ENV : _ref2$NODE_ENV;

  return function () {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    var quantity = args[0],
        unexpected = args.slice(1);

    // validate unless production

    if (NODE_ENV !== 'production') {
      var outerMessage = 'ngettext(' + args.map(String).join(', ') + ')';
      (0, _assert.assertGettextInstance)(gettext, 'ngettext', outerMessage);
      (0, _assert.assertQuantity)(quantity, outerMessage);
      (0, _assert.assertUnexpected)(unexpected, outerMessage);
    }

    return function (strings) {
      for (var _len3 = arguments.length, substitutions = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        substitutions[_key3 - 1] = arguments[_key3];
      }

      // get a msgid template with sensible token names and split by the delimiter
      var tokens = substitutions.map(toToken);
      var msgid = (0, _template.getTemplate)(strings, tokens);
      var msgidForms = splitPlural(msgid);

      // validate unless production
      if (NODE_ENV !== 'production') {
        var innerMessage = 'ngettext(' + args.map(String).join(', ') + ')`' + msgid + '`';
        (0, _assert.assertTokens)(tokens, innerMessage);
        (0, _assert.assertPluralForms)(numPlural, msgidForms.length, innerMessage);
      }

      // translate and make substitutions
      //  we need to call from the gettext parent object in case ngettext() uses 'this'
      var msgstr = gettext.ngettext.apply(gettext, _toConsumableArray(msgidForms).concat([quantity]));
      return (0, _template.makeSubstitutions)({ msgstr: msgstr, tokens: tokens, finaliseToken: finaliseToken });
    };
  };
};

/**
 * Create a hash of all methods using shared options.
 *
 * @param {{gettext:{gettext: function, ngettext: function}, toToken: function,
 * finaliseToken: function, splitPlural: function, numPlural: Number,
 * NODE_ENV: string}} [options] Shared options to pass to method factories
 */
var factory = exports.factory = function factory(options) {
  return {
    gettext: gettextFactory(options),
    ngettext: ngettextFactory(options)
  };
};

/**
 * Gettext as a degenerate template string interpolator.
 * @type {*}
 */
var gettextDefault = exports.gettextDefault = gettextFactory();

/**
 * NGettext as a degenerate template string interpolator.
 * @type {*}
 */
var ngettextDefault = exports.ngettextDefault = ngettextFactory();