'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateCollisions = exports.calculateErrors = exports.defaultFinaliseToken = exports.defaultToToken = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _react = require('react');

/**
 * Convert a substitution into a record.
 *
 * May indicate an `error` but must return reasonable properties regardless.
 *
 * This implementation is opinionated in that:
 *   * non-primitive substitutions must be presented in a {[key]:value} hash
 *   * where a {[key]:value} hash is present, the key must be alphanumeric
 *
 * @param {*} candidate A substitution
 * @param {number} i Array index
 * @return {{error: string, label: string, name: string, key: string, value: *}}
 */
var defaultToToken = exports.defaultToToken = function defaultToToken(candidate, i) {

  // key is for react, only where candidate is a hash with single member
  var isObject = !!candidate && (typeof candidate === 'undefined' ? 'undefined' : _typeof(candidate)) === 'object';
  var fields = isObject ? Object.keys(candidate) : [];
  var key = fields.length === 1 && fields[0] || undefined;

  // label is human readable
  var label = key ? i + ':"' + key + '"' : String(i);

  // name appears in the msgid, it may just be the value of any simple substitution
  // problems can occur if the name has spaces
  var name = key ? '__' + key + '__' : String(candidate);

  // don't clone React elements here, values must be comparable by reference
  var value = key ? candidate[key] : String(candidate);

  // we are opinionated about tokens
  var isKeyRequired = !!candidate && ['object', 'function', 'symbol'].includes(typeof candidate === 'undefined' ? 'undefined' : _typeof(candidate));
  var error = key ? !/^[a-zA-Z0-9_-]+$/.test(key) && 'Keys must be alphanumeric' : isKeyRequired && 'All non-primitive substitutions must be "keyed" by word';

  return { error: error, label: label, name: name, key: key, value: value };
};

/**
 * Convert a token into a final substitution value.
 *
 * React elements are cloned with a unique key. However existing keys are not overwritten.
 *
 * @param {string} key A key implied by the toToken function
 * @param {*} value A substitution value, possibly non-unique
 * @param {number} i The index in which the token will appear, subject to string consolidation
 * @returns {*} A unique value
 */
var defaultFinaliseToken = exports.defaultFinaliseToken = function defaultFinaliseToken(_ref, i) {
  var key = _ref.key,
      value = _ref.value;
  return (0, _react.isValidElement)(value) && !value.key ? (0, _react.cloneElement)(value, { key: key + '-' + i }) : value;
};

/**
 * Calculate a message for those tokens which indicated an error.
 *
 * @param {Array.<{label: string, name: string, value: *}>} tokens A number of tokens
 * @returns {Array.<string>} An array of error statements, possibly empty but never null
 */
var calculateErrors = exports.calculateErrors = function calculateErrors(tokens) {
  return tokens.map(function (_ref2) {
    var error = _ref2.error;
    return error;
  }).filter(Boolean).filter(function (v, i, a) {
    return a.indexOf(v) === i;
  });
};

/**
 * Calculate a message for those tokens with a name collision.
 *
 * @param {Array.<{label: string, name: string, value: *}>} tokens A number of tokens
 * @returns {Array.<string>} An array of collision statements, possibly empty but never null
 */
var calculateCollisions = exports.calculateCollisions = function calculateCollisions(tokens) {
  return tokens.map(function (token, i, arr) {
    var collidesWith = arr.filter(function (other, j) {
      return j === i || other.name === token.name && other.value !== token.value;
    });
    return collidesWith.length > 1 && collidesWith.map(function (_ref3) {
      var label = _ref3.label;
      return label;
    }).join(' vs ');
  }).filter(Boolean).filter(function (msg, i, arr) {
    return arr.indexOf(msg) === i;
  });
};